// emacs: this is -*- c++ -*-
//
//   @file    mapCache.h        
//
//                   
//  
//   Copyright (C) 2016 M.Sutton (sutt@cern.ch)    
//
//   $Id: mapCache.h, v0.0   Thu 14 Jul 2016 13:21:43 CEST sutt $


#ifndef  MAPCACHE_H
#define  MAPCACHE_H

#include <iostream>
#include <vector>
#include <map>



struct mapCache {

public:
  
  /// convenient typedefs 
  
  typedef std::map<double, std::vector<double> >                            map_t; 
  typedef std::map<double, std::vector<double> >::iterator               iterator; 
  typedef std::map<double, std::vector<double> >::const_iterator   const_iterator; 

public:

  mapCache( int nproc=0 ) : 
    mx1(0),  mx2(0),  mQ2(0), 
    mnv(0),  mix(0),
    mx1r(0), mx2r(0),
    morder(0),
    mnproc(nproc),
    mfilled(false)
  { } 

  virtual ~mapCache() { } 

  bool filled() const { return mfilled; }

  void setnproc( int nproc ) { mnproc = nproc; }

  /// "vector-like" operators

  size_t size() const { return mweight.size(); }
  void clear()        { mweight.clear(); }


  /// iterators
  iterator begin() { return mweight.begin(); }
  iterator end()   { return mweight.end(); }

  const_iterator begin() const { return mweight.begin(); }
  const_iterator end()   const { return mweight.end(); }

  

  /// find the weight vector corresponding to this observable value
  /// or else add a new one to this cache

  std::vector<double>& weights(double obs) { 
    iterator itr =  mweight.find( obs );
    if ( itr!=mweight.end() ) return itr->second;
    //    std::cout << "add weights: " << mweight.size() << "\tobs " << obs << std::endl;
    std::pair< iterator, bool > stat = mweight.insert( std::make_pair( obs, std::vector<double>( mnproc, 0 ) ) ) ;
    return stat.first->second;
  }



  /// have any of the kinematic variables changed ?

  bool changed( double obs, 
		int order, double x1, double x2, double Q2, 
	        int nv, 
	        int ix=0, double _x1=0, double _x2=0 ) {
    
    bool bupdate = false;

    if ( mx1!=x1 || mx2!=x2 || mQ2!=Q2 || morder!=order ) bupdate = true;
    
    if ( nv!=mnv ) bupdate = true;
    
    if ( nv>0 ) { 
      if ( ix!=mix || _x1!=mx1r || _x2!=mx2r ) bupdate = true;
    }
    
    return bupdate;

  }


  
  /// update the cached kinematics
    
  void update( double obs, int order, double x1, double x2, double Q2, 
	       int nv, 
	       int ix=0, double _x1=0, double _x2=0 ) {    
    
    mfilled = true;

    mx1 = x1;
    mx2 = x2;
    mQ2 = Q2;
    
    mnv = nv;
    mix = ix;
    
    mx1r = _x1;
    mx2r = _x2;
    
    morder = order;

    clear();
  }



public:

  double  mx1,  mx2, mQ2;
  
  int     mnv;
  int     mix;
  double  mx1r, mx2r;

  double morder;

  int     mnproc;		    

  map_t   mweight;

  bool    mfilled;

};

inline std::ostream& operator<<( std::ostream& s, const mapCache& _p ) { 
  return s;
}


#endif  // MAPCACHE_H 










