//
//   @file    nnlo_utils.cxx         
//   
//
//   @author M.Sutton
// 
//   Copyright (C) 2013 M.Sutton (sutt@cern.ch)    
//
//   $Id: nnlo_utils.cxx, v0.0   Tue 13 Aug 2013 12:20:03 CEST sutt $


#include <cstdlib>
#include <cmath>
#include <string>
#include <fstream>
#include <sys/stat.h>

#include "amconfig.h"

#include "nnlo_utils.h"


/// sadly, includes a return at the end 
std::string nnlo::date() { 
  time_t _t;
  time(&_t);
  std::string a = ctime(&_t);
  std::string b = "";
  for ( unsigned i=0 ; i<a.size()-1 ; i++ ) b+=a[i];
  return b;
}



/// banner
std::string nnlo::banner() {
    std::string version = PACKAGE_VERSION;
    std::string webpage = PACKAGE_URL;
    std::string banner  = "\n";
    banner += "**************************************************************************************\n";
    banner += "*   NNLOJET:     E.W.N.Glover, T.Morgan, ...                 [arXiv:.....]          *\n";
    banner += "*   nnlo-bridge: E.W.N.Glover, C. Gwenlan, T.A.Morgan, K.Rabbertz, M.R.Sutton  ...  *\n";
    banner += "*   APPLgrid:    T. Carli et al.                             [arXiv:0911.2985]       *\n";
    banner += "*   fastNLO:     D. Britzger K.Rabbertz, M. Wobisch. et al. [arXiv:...]            *\n";
    banner += "**************************************************************************************\n";
    banner += "\n";

    banner += "\n";

    banner += "nnlo-bridge version " + std::string(VERSION) + " compiled " + __DATE__ + " " + __TIME__ + "\n";
    banner += "Web page: " + webpage + "\n";
    banner += "\n";
    return banner;
}


/// simple test if a file exists
bool nnlo::file_exists(const std::string& s) {   
  struct stat sb;
  if ( stat( s.c_str(), &sb)==0 ) return true; // && S_ISREG(sb.st_mode ))
  else return false;
}


/// Some kinematical definitions
double nnlo::rapidity(double E, double pz) { 
    return 0.5 * std::log( ( E + pz ) / ( E - pz ) ); 
}
  
double nnlo::pt( double px, double py ) { 
    return std::sqrt( px * px + py * py );
}
  
double nnlo::mass(double E, double px, double py, double pz ) { 
    return std::sqrt( E * E - px * px - py * py - pz * pz );  
}
  


void nnlo::Normalise(TH1D* h) {
  for ( int ibin=1 ; ibin<=h->GetNbinsX() ; ibin++ ) {
    double width = h->GetBinLowEdge(ibin+1) - h->GetBinLowEdge(ibin);
    h->SetBinContent( ibin, h->GetBinContent(ibin)/width );
  }
}


